@@ -18,16 +18,17 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include "modeldna.h"
+#include "modelliemarkov.h"
 
-ModelDNA::ModelDNA(PhyloTree *tree, bool count_rates)
-: ModelGTR(tree, count_rates)
+ModelDNA::ModelDNA(PhyloTree *tree)
+: ModelMarkov(tree)
 {
 }
 
-ModelDNA::ModelDNA(const char *model_name, string model_params, StateFreqType freq, string freq_params, PhyloTree *tree, bool count_rates)
-: ModelGTR(tree, count_rates)
+ModelDNA::ModelDNA(const char *model_name, string model_params, StateFreqType freq, string freq_params, PhyloTree *tree)
+: ModelMarkov(tree)
 {
-	init(model_name, model_params, freq, freq_params);
+  init(model_name, model_params, freq, freq_params);
 }
 
 string getDNAModelInfo(string model_name, string &full_name, string &rate_type, StateFreqType &def_freq) {
@@ -160,18 +161,27 @@ string getDNAModelInfo(string model_name, string &full_name, string &rate_type,
 
 void ModelDNA::init(const char *model_name, string model_params, StateFreqType freq, string freq_params)
 {
-	assert(num_states == 4); // make sure that you create model for DNA
+	ASSERT(num_states == 4); // make sure that you create model for DNA
 	StateFreqType def_freq = FREQ_UNKNOWN;
 	string rate_type;
+	// First try: the time reversible models
 	name = getDNAModelInfo((string)model_name, full_name, rate_type, def_freq);
+	if (name == "") {
+	    // Second try: Lie Markov models. (Note, we're still missing UNREST 
+	    // model. 12.12 is equivalent, but user may not realize that.)
+	    int model_num, symmetry; // returned by getLieMarkovModelInfo, but not used here
+	    ModelLieMarkov::getLieMarkovModelInfo((string)model_name, name, full_name, model_num, symmetry, def_freq);
+	}
 
 	if (name != "") {
 		setRateType(rate_type.c_str());
 	} else {
 		//cout << "User-specified model "<< model_name << endl;
-		if (setRateType(model_name))
-			def_freq = FREQ_ESTIMATE;
-		else {
+	        if (setRateType(model_name)) {
+		    // model was six digits (e.g. 010010 for K2P/HKY)
+		    name = model_name;
+		    full_name = "Time reversible ("+name+")";
+		} else {
 			readParameters(model_name);
 			//name += " (user-defined)";
 		}
@@ -181,25 +191,41 @@ void ModelDNA::init(const char *model_name, string model_params, StateFreqType f
 		readStateFreq(freq_params);
 	}
 	if (model_params != "") {
-		readRates(model_params);
+	  readRates(model_params);
 	}
-
+	
 	if (freq == FREQ_UNKNOWN ||  def_freq == FREQ_EQUAL) freq = def_freq;
-	ModelGTR::init(freq);
+	ModelMarkov::init(freq);
+//    model_parameters = new double [getNDim()+1]; // see setVariables for explaination of +1
+//    setVariables(model_parameters);
+}
+
+void ModelDNA::startCheckpoint() {
+    checkpoint->startStruct("ModelDNA");
 }
 
 void ModelDNA::saveCheckpoint() {
-    checkpoint->startStruct("ModelDNA");
+    // construct model_parameters from rates and base freqs. 
+    // This is one-indexed, so parameters are in model_parameters[1]
+    // up to model_parameters[num_params]
+//    setVariables(model_parameters);
+    startCheckpoint();
     CKP_ARRAY_SAVE(6, rates);
-    checkpoint->endStruct();
-    ModelGTR::saveCheckpoint();
+    endCheckpoint();
+    ModelMarkov::saveCheckpoint();
 }
 
 void ModelDNA::restoreCheckpoint() {
-    ModelGTR::restoreCheckpoint();
-    checkpoint->startStruct("ModelDNA");
+  // curiously, this seems to be the only plase ModelDNA uses model_parameters.
+    ModelMarkov::restoreCheckpoint();
+    startCheckpoint();
     CKP_ARRAY_RESTORE(6, rates);
-    checkpoint->endStruct();
+    endCheckpoint();
+//    getVariables(model_parameters);       // updates rates and state_freq
+    string rate_spec = param_spec;
+    for (auto i = rate_spec.begin(); i != rate_spec.end(); i++)
+        *i = *i + '0';
+    ASSERT(setRateType(rate_spec));
 
     decomposeRateMatrix();
     if (phylo_tree)
@@ -226,7 +252,12 @@ void ModelDNA::readRates(string str) throw(const char*) {
                 num_params++;
                 param_fixed[i+1] = false;
             } else
-                param_fixed[i+1] = true;
+	        if (Params::getInstance().optimize_from_given_params) {
+                    num_params++;
+                    param_fixed[i+1] = false;
+	        } else {
+	            param_fixed[i+1] = true;
+	        }
 			try {
 				rate = convert_double(str.substr(end_pos).c_str(), new_end_pos);
 			} catch (string str) {
@@ -268,11 +299,11 @@ string ModelDNA::getNameParams() {
 	return retname.str();
 }
 
-bool ModelDNA::setRateType(const char *rate_str) {
+bool ModelDNA::setRateType(string rate_str) {
 	//char first_type = 127;
 	//char last_type = 0;
 	//char t = first_type;
-	int num_ch = strlen(rate_str);
+	int num_ch = rate_str.length();
 	int i;
 
 	if (num_ch != getNumRateEntries()) {
@@ -317,7 +348,7 @@ bool ModelDNA::setRateType(const char *rate_str) {
 		}
 	}
 
-	assert(param_spec.length() == num_ch);
+	ASSERT(param_spec.length() == num_ch);
 	double *avg_rates = new double[num_params+1];
 	int *num_rates = new int[num_params+1];
 	memset(avg_rates, 0, sizeof(double) * (num_params+1));
@@ -346,10 +377,15 @@ bool ModelDNA::setRateType(const char *rate_str) {
 
 
 int ModelDNA::getNDim() {
-	assert(freq_type != FREQ_UNKNOWN);
-	int ndim = num_params; 
+	ASSERT(freq_type != FREQ_UNKNOWN);
+	// possible TO-DO: cache nFreqParams(freq_type) to avoid repeat calls.
+//        return (num_params+nFreqParams(freq_type));
+
+	int ndim = num_params;
 	if (freq_type == FREQ_ESTIMATE) 
 		ndim += num_states-1;
+    else
+        ndim += nFreqParams(freq_type);
 	return ndim;
 }
 
@@ -369,68 +405,93 @@ void ModelDNA::writeParameters(ostream &out) {
 	}
 }
 
-
+/*
+ * getVariables *changes* the state of the model, setting from *variables
+ * Returns true if the model state has changed, false if not.
+ */
 bool ModelDNA::getVariables(double *variables) {
-	int i;
+    int i;
     bool changed = false;
-	if (num_params > 0) {
-		int num_all = param_spec.length();
-		if (verbose_mode >= VB_MAX) {
-			for (i = 1; i <= num_params; i++)
-				cout << "  estimated variables[" << i << "] = " << variables[i] << endl;
-		}
-		for (i = 0; i < num_all; i++)
-			if (!param_fixed[param_spec[i]]) {
+    if (num_params > 0) {
+        int num_all = param_spec.length();
+        if (verbose_mode >= VB_MAX) {
+            for (i = 1; i <= num_params; i++)
+                cout << "  estimated variables[" << i << "] = " << variables[i] << endl;
+        }
+        for (i = 0; i < num_all; i++)
+            if (!param_fixed[param_spec[i]]) {
                 changed |= (rates[i] != variables[(int)param_spec[i]]);
-				rates[i] = variables[(int)param_spec[i]];
-			}
-	}
+                            rates[i]  = variables[(int)param_spec[i]];
+             }
+   
+    }
 	if (freq_type == FREQ_ESTIMATE) {
         // 2015-09-07: relax the sum of state_freq to be 1, this will be done at the end of optimization
 		int ndim = getNDim();
 		changed |= memcmpcpy(state_freq, variables+(ndim-num_states+2), (num_states-1)*sizeof(double));
-//		double sum = 0;
-//		for (i = 0; i < num_states-1; i++) 
-//			sum += state_freq[i];
-//		state_freq[num_states-1] = 1.0 - sum;
+                double sum = 0;
+                for (i = 0; i < num_states-1; i++) 
+                        sum += state_freq[i];
+                state_freq[num_states-1] = 1.0 - sum;
+    } else {
+        // BQM: for special DNA freq stuffs from MDW
+        changed |= freqsFromParams(state_freq,variables+num_params+1,freq_type);
+    }
+    return changed;
+
 
         // BUG FIX 2015.08.28
 //        int nrate = getNDim();
 //        if (freq_type == FREQ_ESTIMATE) nrate -= (num_states-1);
-//		double sum = 1.0;
-//		int i, j;
-//		for (i = 1; i < num_states; i++)
-//			sum += variables[nrate+i];
-//		for (i = 0, j = 1; i < num_states; i++)
-//			if (i != highest_freq_state) {
-//				state_freq[i] = variables[nrate+j] / sum;
-//				j++;
-//			}
-//		state_freq[highest_freq_state] = 1.0/sum;
-	}
-    return changed;
+//              double sum = 1.0;
+//              int i, j;
+//              for (i = 1; i < num_states; i++)
+//                      sum += variables[nrate+i];
+//              for (i = 0, j = 1; i < num_states; i++)
+//                      if (i != highest_freq_state) {
+//                              state_freq[i] = variables[nrate+j] / sum;
+//                              j++;
+//                      }
+//              state_freq[highest_freq_state] = 1.0/sum;
 }
 
+/*
+ * setVariables *reads* the state of the model and writes into "variables"
+ * Model does not change state. *variables should have length getNDim()+1
+ * If param_spec is (e.g.) 012210 (e.g. K3P model) then in general
+ * we'd have rates 0 and 5 (A<->C and G<->T) written to variables[0],
+ * rates 1 and 4 to variables[1] and rates 2 and 3 to variables[2].
+ * However one of these (typically 0) is 'fixed' (param_fixed)
+ * to always have value 1, and this doesn't get written. 
+ * num_parameters in this case will be two, for two free rates parameters.
+ * Base frequency parameters get written after the rate parameters, so
+ * K3P+FO model (i.e. fits base frequencies with no constraints) would
+ * use variables[3] to variables[5] (3 values) to store base freq info. 
+ */
 void ModelDNA::setVariables(double *variables) {
-	if (num_params > 0) {
-		int num_all = param_spec.length();
-		for (int i = 0; i < num_all; i++)
-			if (!param_fixed[param_spec[i]])
-				variables[(int)param_spec[i]] = rates[i];
-	}
+    if (num_params > 0) {
+        int num_all = param_spec.length();
+        for (int i = 0; i < num_all; i++)
+            if (!param_fixed[param_spec[i]])
+                variables[(int)param_spec[i]] = rates[i];
+    }
+    // and copy parameters for base frequencies
+
 	if (freq_type == FREQ_ESTIMATE) {
         // 2015-09-07: relax the sum of state_freq to be 1, this will be done at the end of optimization
 		int ndim = getNDim();
 		memcpy(variables+(ndim-num_states+2), state_freq, (num_states-1)*sizeof(double));
+    } else {
+        paramsFromFreqs(variables+num_params+1, state_freq, freq_type);
+    }
 
         // BUG FIX 2015.08.28
 //        int nrate = getNDim();
 //        if (freq_type == FREQ_ESTIMATE) nrate -= (num_states-1);
-//		int i, j;
-//		for (i = 0, j = 1; i < num_states; i++)
-//			if (i != highest_freq_state) {
-//				variables[nrate+j] = state_freq[i] / state_freq[highest_freq_state];
-//				j++;
-//			}
-	}
+//              int i, j;
+//              for (i = 0, j = 1; i < num_states; i++)
+//                      if (i != highest_freq_state) {
+//                              variables[nrate+j] = state_freq[i] / state_freq[highest_freq_state];
+//                              j++;
+//                      }
 }
