@@ -8,7 +8,7 @@
 #ifndef MODELMORPHOLOGY_H_
 #define MODELMORPHOLOGY_H_
 
-#include "modelgtr.h"
+#include "modelmarkov.h"
 
 /**
  * This class implement ML model for morphological data. Such models are:
@@ -16,7 +16,7 @@
  * - ORDERED: allowing only transition from state i to i-1 and i+1
  * TODO: Mkv to account for absence of constant sites
  */
-class ModelMorphology: public ModelGTR {
+class ModelMorphology: public ModelMarkov {
 public:
 	/**
 		constructor
@@ -26,6 +26,7 @@ public:
 	*/
 	ModelMorphology(const char *model_name, string model_params, StateFreqType freq, string freq_params, PhyloTree *tree);
 
+
 	/**
 		initialization, called automatically by the constructor, no need to call it
 		@param model_name model name, e.g., JC, HKY.
@@ -33,12 +34,22 @@ public:
 	*/
 	virtual void init(const char *model_name, string model_params, StateFreqType freq, string freq_params);
 
+    /**
+        start structure for checkpointing
+    */
+    virtual void startCheckpoint();
 
 	/**
 		return the number of dimensions
 	*/
 	virtual int getNDim() { return 0; }
 
+	/**
+		read the rates from an input stream. it will throw error messages if failed
+		@param in input stream
+	*/
+	virtual void readRates(istream &in) throw(const char*, string);
+
     virtual ~ModelMorphology();
 };
 
