@@ -19,51 +19,29 @@
 
 #include "modelset.h"
 
-ModelSet::ModelSet(const char *model_name, PhyloTree *tree) : ModelGTR(tree)
+ModelSet::ModelSet(const char *model_name, PhyloTree *tree) : ModelMarkov(tree)
 {
 	name = full_name = model_name;
 	name += "+SSF";
 	full_name += "+site-specific state-frequency model (unpublished)";
 }
 
-void ModelSet::computeTransMatrix(double time, double* trans_matrix)
+void ModelSet::computeTransMatrix(double time, double* trans_matrix, int mixture)
 {
     // TODO not working with vectorization
-    assert(0);
+    ASSERT(0);
 	for (iterator it = begin(); it != end(); it++) {
-		(*it)->computeTransMatrix(time, trans_matrix);
+		(*it)->computeTransMatrix(time, trans_matrix, mixture);
 		trans_matrix += (num_states * num_states);
 	}
 }
 
-void ModelSet::computeTransMatrixFreq(double time, double* trans_matrix)
+void ModelSet::computeTransDerv(double time, double* trans_matrix, double* trans_derv1, double* trans_derv2, int mixture)
 {
     // TODO not working with vectorization
-    assert(0);
+    ASSERT(0);
 	for (iterator it = begin(); it != end(); it++) {
-		(*it)->computeTransMatrixFreq(time, trans_matrix);
-		trans_matrix += (num_states * num_states);
-	}
-}
-
-void ModelSet::computeTransDerv(double time, double* trans_matrix, double* trans_derv1, double* trans_derv2)
-{
-    // TODO not working with vectorization
-    assert(0);
-	for (iterator it = begin(); it != end(); it++) {
-		(*it)->computeTransDerv(time, trans_matrix, trans_derv1, trans_derv2);
-		trans_matrix += (num_states * num_states);
-		trans_derv1 += (num_states * num_states);
-		trans_derv2 += (num_states * num_states);
-	}
-}
-
-void ModelSet::computeTransDervFreq(double time, double rate_val, double* trans_matrix, double* trans_derv1, double* trans_derv2)
-{
-    // TODO not working with vectorization
-    assert(0);
-	for (iterator it = begin(); it != end(); it++) {
-		(*it)->computeTransDervFreq(time, rate_val, trans_matrix, trans_derv1, trans_derv2);
+		(*it)->computeTransDerv(time, trans_matrix, trans_derv1, trans_derv2, mixture);
 		trans_matrix += (num_states * num_states);
 		trans_derv1 += (num_states * num_states);
 		trans_derv2 += (num_states * num_states);
@@ -72,8 +50,8 @@ void ModelSet::computeTransDervFreq(double time, double rate_val, double* trans_
 
 int ModelSet::getPtnModelID(int ptn)
 {
-	assert(ptn >= 0 && ptn < pattern_model_map.size());
-	assert(pattern_model_map[ptn] >= 0 && pattern_model_map[ptn] < size());
+	ASSERT(ptn >= 0 && ptn < pattern_model_map.size());
+	ASSERT(pattern_model_map[ptn] >= 0 && pattern_model_map[ptn] < size());
     return pattern_model_map[ptn];
 }
 
@@ -127,7 +105,7 @@ double ModelSet::computeTrans(double time, int model_id, int state1, int state2,
 
 int ModelSet::getNDim()
 {
-	assert(size());
+	ASSERT(size());
     return front()->getNDim();
 }
 
@@ -197,7 +175,7 @@ void ModelSet::decomposeRateMatrix()
 
 bool ModelSet::getVariables(double* variables)
 {
-	assert(size());
+	ASSERT(size());
     bool changed = false;
 	for (iterator it = begin(); it != end(); it++)
 		changed |= (*it)->getVariables(variables);
@@ -206,7 +184,7 @@ bool ModelSet::getVariables(double* variables)
 
 void ModelSet::setVariables(double* variables)
 {
-	assert(size());
+	ASSERT(size());
 	front()->setVariables(variables);
 }
 
