@@ -18,7 +18,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include "partitionmodel.h"
-#include "superalignment.h"
+#include "alignment/superalignment.h"
 #include "model/rategamma.h"
 
 PartitionModel::PartitionModel()
@@ -41,7 +41,7 @@ PartitionModel::PartitionModel(Params &params, PhyloSuperTree *tree, ModelsBlock
 	site_rate = new RateHeterogeneity();
 	site_rate->setTree(tree);
 
-    string model_name = params.model_name;
+//    string model_name = params.model_name;
     PhyloSuperTree::iterator it;
     int part;
     if (params.link_alpha) {
@@ -49,14 +49,14 @@ PartitionModel::PartitionModel(Params &params, PhyloSuperTree *tree, ModelsBlock
         linked_alpha = params.gamma_shape;
     }
     for (it = tree->begin(), part = 0; it != tree->end(); it++, part++) {
-        assert(!((*it)->getModelFactory()));
-        params.model_name = tree->part_info[part].model_name;
-        if (params.model_name == "") // if empty, take model name from command option
-        	params.model_name = model_name;
-        (*it)->setModelFactory(new ModelFactory(params, (*it), models_block));
+        ASSERT(!((*it)->getModelFactory()));
+        string model_name = tree->part_info[part].model_name;
+        if (model_name == "") // if empty, take model name from command option
+        	model_name = params.model_name;
+        (*it)->setModelFactory(new ModelFactory(params, model_name, (*it), models_block));
         (*it)->setModel((*it)->getModelFactory()->model);
         (*it)->setRate((*it)->getModelFactory()->site_rate);
-        params.model_name = model_name;
+//        params.model_name = model_name;
         if ((*it)->aln->getNSeq() < tree->aln->getNSeq() && (*it)->getModel()->freq_type == FREQ_EMPIRICAL && (*it)->aln->seq_type != SEQ_CODON) {
         	// modify state_freq to account for empty sequences
         	(*it)->aln->computeStateFreq((*it)->getModel()->state_freq, (*it)->aln->getNSite() * (tree->aln->getNSeq() - (*it)->aln->getNSeq()));
@@ -76,8 +76,12 @@ void PartitionModel::setCheckpoint(Checkpoint *checkpoint) {
 		(*it)->getModelFactory()->setCheckpoint(checkpoint);
 }
 
-void PartitionModel::saveCheckpoint() {
+void PartitionModel::startCheckpoint() {
     checkpoint->startStruct("PartitionModel");
+}
+
+void PartitionModel::saveCheckpoint() {
+    startCheckpoint();
     CKP_SAVE(linked_alpha);
     PhyloSuperTree *tree = (PhyloSuperTree*)site_rate->getTree();
     int part = 0;
@@ -86,14 +90,14 @@ void PartitionModel::saveCheckpoint() {
         (*it)->getModelFactory()->saveCheckpoint();
         checkpoint->endStruct();
     }
-    checkpoint->endStruct();
+    endCheckpoint();
 
     CheckpointFactory::saveCheckpoint();
 }
 
 void PartitionModel::restoreCheckpoint() {
     CheckpointFactory::restoreCheckpoint();
-    checkpoint->startStruct("PartitionModel");
+    startCheckpoint();
     CKP_RESTORE(linked_alpha);
 
     PhyloSuperTree *tree = (PhyloSuperTree*)site_rate->getTree();
@@ -104,14 +108,14 @@ void PartitionModel::restoreCheckpoint() {
         checkpoint->endStruct();
     }
 
-    checkpoint->endStruct();
+    endCheckpoint();
 }
 
-int PartitionModel::getNParameters() {
+int PartitionModel::getNParameters(int brlen_type) {
     PhyloSuperTree *tree = (PhyloSuperTree*)site_rate->getTree();
 	int df = 0;
     for (PhyloSuperTree::iterator it = tree->begin(); it != tree->end(); it++) {
-    	df += (*it)->getModelFactory()->getNParameters();
+    	df += (*it)->getModelFactory()->getNParameters(brlen_type);
     }
     if (linked_alpha > 0)
         df ++;
@@ -162,7 +166,7 @@ double PartitionModel::optimizeParameters(int fixed_len, bool write_info, double
         {
     		cout << "Optimizing " << tree->at(part)->getModelName() <<
         		" parameters for partition " << tree->part_info[part].name <<
-        		" (" << tree->at(part)->getModelFactory()->getNParameters() << " free parameters)" << endl;
+        		" (" << tree->at(part)->getModelFactory()->getNParameters(fixed_len) << " free parameters)" << endl;
         }
         tree_lh += tree->at(part)->getModelFactory()->optimizeParameters(fixed_len, write_info && verbose_mode >= VB_MED, 
             logl_epsilon/min(ntrees,10), gradient_epsilon/min(ntrees,10));
@@ -172,6 +176,8 @@ double PartitionModel::optimizeParameters(int fixed_len, bool write_info, double
     if (tree->params->link_alpha) {
         tree_lh = optimizeLinkedAlpha(write_info, gradient_epsilon);
     }
+	if (verbose_mode >= VB_MED || write_info)
+		cout << "Optimal log-likelihood: " << tree_lh << endl;
     return tree_lh;
 }
 
@@ -194,7 +200,7 @@ double PartitionModel::optimizeParametersGammaInvar(int fixed_len, bool write_in
         {
     		cout << "Optimizing " << tree->at(part)->getModelName() <<
         		" parameters for partition " << tree->part_info[part].name <<
-        		" (" << tree->at(part)->getModelFactory()->getNParameters() << " free parameters)" << endl;
+        		" (" << tree->at(part)->getModelFactory()->getNParameters(fixed_len) << " free parameters)" << endl;
         }
         tree_lh += tree->at(part)->getModelFactory()->optimizeParametersGammaInvar(fixed_len, write_info && verbose_mode >= VB_MED, 
             logl_epsilon/min(ntrees,10), gradient_epsilon/min(ntrees,10));
@@ -204,6 +210,8 @@ double PartitionModel::optimizeParametersGammaInvar(int fixed_len, bool write_in
     if (tree->params->link_alpha) {
         tree_lh = optimizeLinkedAlpha(write_info, gradient_epsilon);
     }
+	if (verbose_mode >= VB_MED || write_info)
+		cout << "Optimal log-likelihood: " << tree_lh << endl;
     return tree_lh;
 }
 
