@@ -5,7 +5,7 @@
  *      Author: minh
  */
 
-#include "phylotree.h"
+#include "tree/phylotree.h"
 #include "ratefree.h"
 #include "rateinvar.h"
 
@@ -24,7 +24,7 @@ const double MIN_FREE_RATE_PROP = 0.001;
 const double MAX_FREE_RATE_PROP = 1000;
 
 RateFree::RateFree(int ncat, double start_alpha, string params, bool sorted_rates, string opt_alg, PhyloTree *tree) : RateGamma(ncat, start_alpha, false, tree) {
-	fix_params = false;
+	fix_params = 0;
 	prop = NULL;
     this->sorted_rates = sorted_rates;
     optimizing_params = 0;
@@ -35,46 +35,60 @@ RateFree::RateFree(int ncat, double start_alpha, string params, bool sorted_rate
 	DoubleVector params_vec;
 	try {
 		convert_double_vec(params.c_str(), params_vec);
-		if (params_vec.size() != ncategory*2)
-			outError("Number of parameters for FreeRate model must be twice number of categories");
 		int i;
 		double sum, sum_prop;
-		for (i = 0, sum = 0.0, sum_prop = 0.0; i < ncategory; i++) {
-			prop[i] = params_vec[i*2];
-			rates[i] = params_vec[i*2+1];
-			sum += prop[i]*rates[i];
-			sum_prop += prop[i];
-		}
+        if (params_vec.size() == ncategory) {
+            // only inputing prop
+            for (i = 0, sum_prop = 0.0; i < ncategory; i++) {
+                prop[i] = params_vec[i];
+                rates[i] = 1.0;
+                sum_prop += prop[i];
+            }
+            fix_params = 1;
+        } else {
+            if (params_vec.size() != ncategory*2)
+                outError("Number of parameters for FreeRate model must be twice number of categories");
+            for (i = 0, sum = 0.0, sum_prop = 0.0; i < ncategory; i++) {
+                prop[i] = params_vec[i*2];
+                rates[i] = params_vec[i*2+1];
+                sum += prop[i]*rates[i];
+                sum_prop += prop[i];
+            }
+            for (i = 0; i < ncategory; i++)
+                rates[i] /= sum;
+            fix_params = 2;
+        }
 		if (fabs(sum_prop-1.0) > 1e-5)
 			outError("Sum of category proportions not equal to 1");
-		for (i = 0; i < ncategory; i++)
-			rates[i] /= sum;
-		fix_params = true;
 	} catch (string &str) {
 		outError(str);
 	}
 }
 
+void RateFree::startCheckpoint() {
+    checkpoint->startStruct("RateFree" + convertIntToString(ncategory));
+}
+
 void RateFree::saveCheckpoint() {
-    checkpoint->startStruct("RateFree");
+    startCheckpoint();
 //    CKP_SAVE(fix_params);
 //    CKP_SAVE(sorted_rates);
 //    CKP_SAVE(optimize_alg);
     CKP_ARRAY_SAVE(ncategory, prop);
     CKP_ARRAY_SAVE(ncategory, rates);
-    checkpoint->endStruct();
-    RateGamma::saveCheckpoint();
+    endCheckpoint();
+//    RateGamma::saveCheckpoint();
 }
 
 void RateFree::restoreCheckpoint() {
-    RateGamma::restoreCheckpoint();
-    checkpoint->startStruct("RateFree");
+//    RateGamma::restoreCheckpoint();
+    startCheckpoint();
 //    CKP_RESTORE(fix_params);
 //    CKP_RESTORE(sorted_rates);
 //    CKP_RESTORE(optimize_alg);
     CKP_ARRAY_RESTORE(ncategory, prop);
     CKP_ARRAY_RESTORE(ncategory, rates);
-    checkpoint->endStruct();
+    endCheckpoint();
 
 //	setNCategory(ncategory);
 }
@@ -109,24 +123,34 @@ void RateFree::setNCategory(int ncat) {
 	full_name += " with " + convertIntToString(ncategory) + " categories";
 }
 
-void RateFree::setRateAndProp(RateFree *input) {
-    assert(input->ncategory == ncategory-1);
-    setPInvar(input->getPInvar());
-    int k = 0, i;
+void RateFree::initFromCatMinusOne() {
+    ncategory--;
+    restoreCheckpoint();
+    ncategory++;
+
+    int first = 0, second = -1, i;
     // get the category k with largest proportion
     for (i = 1; i < ncategory-1; i++)
-        if (input->prop[i] > input->prop[k]) k = i;
+        if (prop[i] > prop[first]) {
+            first = i;
+        }
+    second = (first == 0) ? 1 : 0;
+    for (i = 0; i < ncategory-1; i++)
+        if (prop[i] > prop[second] && second != first)
+            second = i;
 
-    memcpy(rates, input->rates, (k+1)*sizeof(double));
-    memcpy(prop, input->prop, (k+1)*sizeof(double));
-    rates[k+1] = 1.414*rates[k]; // sqrt(2)
-    prop[k+1] = prop[k]/2;
-    rates[k] = 0.586*rates[k];
-    prop[k] = prop[k]/2;
-    if (k < ncategory-2) {
-        memcpy(&rates[k+2], &input->rates[k+1], (ncategory-2-k)*sizeof(double));
-        memcpy(&prop[k+2], &input->prop[k+1], (ncategory-2-k)*sizeof(double));
-    }
+//    memmove(rates, input->rates, (k+1)*sizeof(double));
+//    memmove(prop, input->prop, (k+1)*sizeof(double));
+
+    // divide highest category into 2 of the same prop
+    rates[ncategory-1] = (-rates[second] + 3*rates[first])/2.0;
+    prop[ncategory-1] = prop[first]/2;
+    rates[first] = (rates[second]+rates[first])/2.0;
+    prop[first] = prop[first]/2;
+//    if (k < ncategory-2) {
+//        memcpy(&rates[k+2], &input->rates[k+1], (ncategory-2-k)*sizeof(double));
+//        memcpy(&prop[k+2], &input->prop[k+1], (ncategory-2-k)*sizeof(double));
+//    }
     // copy half of k to the last category
 
 
@@ -175,7 +199,9 @@ double RateFree::rescaleRates() {
 }
 
 int RateFree::getNDim() { 
-    if (fix_params) return 0;
+    if (fix_params == 2) return 0;
+    if (fix_params == 1) // only fix prop
+        return (ncategory-1);
     if (optimizing_params == 0) return (2*ncategory-2); 
     if (optimizing_params == 1) // rates
         return ncategory-1;
@@ -211,7 +237,8 @@ double RateFree::optimizeParameters(double gradient_epsilon) {
 
     // TODO: turn off EM algorithm for +ASC model
     if ((optimize_alg.find("EM") != string::npos && phylo_tree->getModelFactory()->unobserved_ptns.empty()))
-        return optimizeWithEM();
+        if (fix_params == 0)
+            return optimizeWithEM();
 
 	//if (freq_type == FREQ_ESTIMATE) scaleStateFreq(false);
 
@@ -224,6 +251,8 @@ double RateFree::optimizeParameters(double gradient_epsilon) {
 //    score = optimizeWeights();
 
     int left = 1, right = 2;
+    if (fix_params == 1) // fix proportions
+        right = 1;
     if (optimize_alg.find("1-BFGS") != string::npos) {
         left = 0; 
         right = 0;
@@ -490,10 +519,10 @@ double RateFree::optimizeWithEM() {
             phylo_tree->printTree(cout, WT_BR_LEN+WT_NEWLINE);
             writeInfo(cout);
         }
-        assert(score < 0);
+        ASSERT(score < 0);
         
         if (step > 0)
-            assert(score > old_score-0.1);
+            ASSERT(score > old_score-0.1);
             
         old_score = score;
         
@@ -507,7 +536,7 @@ double RateFree::optimizeWithEM() {
             for (c = 0; c < nmix; c++) {
                 lk_ptn += this_lk_cat[c];
             }
-            assert(lk_ptn != 0.0);
+            ASSERT(lk_ptn != 0.0);
             lk_ptn = phylo_tree->ptn_freq[ptn] / lk_ptn;
             
             // transform _pattern_lh_cat into posterior probabilities of each category
@@ -518,7 +547,7 @@ double RateFree::optimizeWithEM() {
             
         } 
         
-        // M-step, update weights according to (*)        
+        // M-step, update weights according to (*)
         int maxpropid = 0;
         double new_pinvar = 0.0;    
         for (c = 0; c < nmix; c++) {
@@ -537,7 +566,7 @@ double RateFree::optimizeWithEM() {
         }
         // break if some probabilities too small
         if (zero_prop) break;
-        
+
         bool converged = true;
         double sum_prop = 0.0;
         for (c = 0; c < nmix; c++) {
@@ -553,26 +582,28 @@ double RateFree::optimizeWithEM() {
 
         if (new_pinvar > 1e-4 && getPInvar() != 0.0) {
             converged = converged && (fabs(getPInvar()-new_pinvar) < 1e-4);
+            if (isFixPInvar())
+                outError("Fixed given p-invar is not supported");
             setPInvar(new_pinvar);
 //            setOptimizePInvar(false);
             phylo_tree->computePtnInvar();
         }
         
-        assert(fabs(sum_prop+new_pinvar-1.0) < MIN_PROP);
+        ASSERT(fabs(sum_prop+new_pinvar-1.0) < MIN_PROP);
         
         // now optimize rates one by one
         double sum = 0.0;
         for (c = 0; c < nmix; c++) {
             tree->copyPhyloTree(phylo_tree);
-            ModelGTR *subst_model;
+            ModelMarkov *subst_model;
             if (phylo_tree->getModel()->isMixture() && phylo_tree->getModelFactory()->fused_mix_rate)
-                subst_model = ((ModelMixture*)phylo_tree->getModel())->at(c);
+                subst_model = (ModelMarkov*)phylo_tree->getModel()->getMixtureClass(c);
             else
-                subst_model = (ModelGTR*)phylo_tree->getModel();
+                subst_model = (ModelMarkov*)phylo_tree->getModel();
             tree->setModel(subst_model);
             subst_model->setTree(tree);
             model_fac->model = subst_model;
-            if (subst_model->isMixture() || subst_model->isSiteSpecificModel())
+            if (subst_model->isMixture() || subst_model->isSiteSpecificModel() || !subst_model->isReversible())
                 tree->setLikelihoodKernel(phylo_tree->sse, phylo_tree->num_threads);
 
                         
@@ -599,6 +630,10 @@ double RateFree::optimizeWithEM() {
         if (converged) break;
     }
     
+        // sort the rates in increasing order
+    if (sorted_rates)
+        quicksort(rates, 0, ncategory-1, prop);
+    
     // deattach memory
 //    tree->central_partial_lh = NULL;
 //    tree->central_scale_num = NULL;
