@@ -21,6 +21,11 @@ public:
 	*/
     RateFreeInvar(int ncat, double start_alpha, string params, bool sorted_rates, double p_invar_sites, string opt_alg, PhyloTree *tree);
 
+    /**
+        start structure for checkpointing
+    */
+    virtual void startCheckpoint();
+
     /**
         save object into the checkpoint
     */
